package ch.ethz.ssh2.crypto.dh;


import ch.ethz.ssh2.crypto.digest.HashForSSH2Types;
import java.math.BigInteger;
import java.security.SecureRandom;

/**
 * DhExchange.
 * 
 * @author Christian Plattner, plattner@inf.ethz.ch
 * @version $Id: DhExchange.java,v 1.1 2005/05/26 14:53:24 cplattne Exp $
 */
public class DhExchange
{
	/* Given by the standard */

	static final BigInteger p;
	static final BigInteger g;

	/* Client public and private */

	BigInteger e;
	BigInteger x;

	/* Server public */

	BigInteger f;

	/* Shared secret */

	BigInteger k;

	static
	{
		final String p_string = "17976931348623159077083915679378745319786029604875"
				+ "60117064444236841971802161585193689478337958649255415021805654859805036464"
				+ "40548199239100050792877003355816639229553136239076508735759914822574862575"
				+ "00742530207744771258955095793777842444242661733472762929938766870920560605"
				+ "0270810842907692932019128194467627007";

		p = new BigInteger(p_string);
		g = new BigInteger("2");
	}

	public DhExchange()
	{
	}

	public void init()
	{
		k = null;

		x = new BigInteger(p.bitLength() - 1, new SecureRandom());
		e = g.modPow(x, p);
	}

	/**
	 * @return Returns the e.
	 * @throws IllegalStateException
	 */
	public BigInteger getE()
	{
		if (e == null)
			throw new IllegalStateException("DhDsaExchange not initialized!");

		return e;
	}

	/**
	 * @return Returns the shared secret k.
	 * @throws IllegalStateException
	 */
	public BigInteger getK()
	{
		if (k == null)
			throw new IllegalStateException(
					"Shared secret not yet known, need f first!");

		return k;
	}

	/**
	 * @param f
	 */
	public void setF(BigInteger f)
	{
		if (e == null) throw new IllegalStateException("DhDsaExchange not initialized!");

		BigInteger zero = BigInteger.valueOf(0);

		if (zero.compareTo(f) >= 0 || p.compareTo(f) <= 0)
			throw new IllegalArgumentException("Invalid f specified!");

		this.f = f;
		this.k = f.modPow(x, p);
	}

	public byte[] calculateH(byte[] clientversion, byte[] serverversion,
			byte[] clientKexPayload, byte[] serverKexPayload, byte[] hostKey)
	{
		HashForSSH2Types hash = new HashForSSH2Types("SHA");

		hash.updateByteString(clientversion);
		hash.updateByteString(serverversion);
		hash.updateByteString(clientKexPayload);
		hash.updateByteString(serverKexPayload);
		hash.updateByteString(hostKey);
		hash.updateBigInt(e);
		hash.updateBigInt(f);
		hash.updateBigInt(k);

		return hash.getDigest();
	}
}
